﻿using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using Chambills.Present.Core;
using NUnit.Framework;

namespace Chambills.Present.Validation.Tests
{
    [TestFixture]
    public class AttributeBasedViewModelValidatorContributorFixture
    {


        private AttributeBasedViewModelValidatorContributor validator;
        private TestViewModel model;


        [SetUp]
        public void Setup()
        {
            model = new TestViewModel();
            validator = new AttributeBasedViewModelValidatorContributor { Model = model };
        }

       
        

        [Test]
        public void ValidFieldsProduceNoError()
        {
            model.RequiredField = "test";
            Assert.IsTrue(validator.GetCurrentErrors().IsEmpty(), "Error Message should be empty");
        }

        [Test]
        public void FieldsAreValidated()
        {
            model.RequiredField = null;
            Assert.IsFalse(validator.GetCurrentErrors().IsEmpty());
        }

        [Test]
        public void CorrectMessageReturned()
        {
            model.RequiredField = null;
            Assert.AreEqual(TestViewModel.RequiredMsg, validator.GetCurrentErrors().FirstOrDefault());
        }
    }

    

   

    public class TestViewModel : NotifiableObject, IViewModel
    {
        public const string RequiredMsg = "Required Field";

        private string requiredField;

     

        [Required(ErrorMessage = RequiredMsg)]
        public virtual string RequiredField
        {
            get { return requiredField; }
            set
            {
                requiredField = value;
                OnPropertyChanged(()=>RequiredField);
            }
        }
    }
}
