﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using NUnit.Framework;

namespace Chambills.Present.Validation.Tests
{
    [TestFixture, RequiresSTA]
    public class BrokenBindingNotifierFixture
    {
        private BrokenBindingNotifier notifier;
        private StackPanel parent;
        private BindingExpressionBase binding;
        private const string BindingPath = "Text";

        [SetUp]
        public void Setup()
        {
            notifier = new BrokenBindingNotifier();
            parent = new StackPanel();
            parent.Children.Add(notifier);
            binding = BindingOperations.SetBinding(parent, FrameworkElement.TagProperty, new ValidatedBinding(BindingPath));


        }

        [Test]
        public void PropertiesWithBrokenBindingsContainBrokenPropertyNameAfterError()
        {

            RaiseError();
            CollectionAssert.Contains(notifier.PropertiesWithBrokenBindings, BindingPath);
        }


        [Test]
        public void PropertiesWithBrokenBindingsDoesNotContainBrokenPropertyNameWhenErrorFixed()
        {

            RaiseError();
            RemoveError();

            CollectionAssert.DoesNotContain(notifier.PropertiesWithBrokenBindings, BindingPath);
        }

        private void RaiseError()
        {
            System.Windows.Controls.Validation.MarkInvalid(binding, new ValidationError(new ExceptionValidationRule(), binding));
        }

        private void RemoveError()
        {
            System.Windows.Controls.Validation.ClearInvalid(binding);
        }

    }
}
